@@ -1,8 +1,8 @@
   /*
 
-  knetfilter v. 2.0.3
+  knetfilter v. 2.1.2
 
-  Copyright (C) 2000 Luigi Genoni
+  Copyright (C) 2001 Luigi Genoni
   		     venom@DarkStar.sns.it
   */
 
@@ -136,8 +136,7 @@ public slots:
   void delmarkDone();
   void markchainActivated(int);
   void markpolicyActivated(int);
-  void markflagsActivated(int);
-  
+  void markstateActivated(int);
 
   /* interface functions */
   void startprobeInterface();
@@ -318,6 +317,7 @@ private:
   QComboBox *markchain;
   QComboBox *markpolicy;
   QComboBox *markflags;
+  QComboBox *markstate;
   QComboBox *nat;
   QComboBox *natinterface;
   QComboBox *natinterfacenr;
@@ -382,6 +382,9 @@ private:
   char *markchainChoice;
   char *markpolicyChoice;
   char *markflagChoice;
+  char *markstate_modules;
+  char *markstate_definition;
+  char *markstateChoice; 
   char *natprotocolChoice;
   char *polchainChoice;
   char *chainpolicyChoice;
