@@ -369,7 +369,7 @@ void knetfilter::masqRules() {
   if ( masqWin == NULL )
     {
       
-	  setupConnect();
+//      setupConnect();
 
       masqWin = new KNFWidget();
       masqWin->resize(640,310);
