@@ -1,8 +1,8 @@
   /*
 
-  knetfilter v 2.0.0
+  knetfilter v 2.1.1
 
-  Copyright (C) 2000 Luigi Genoni
+  Copyright (C) 2001 Luigi Genoni
   		     venom@DarkStar.sns.it
   */
   
@@ -28,9 +28,15 @@ void knetfilter::addRule() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("255.255.255.255");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
     
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
 	extramask.sprintf("255.255.255.255");
@@ -47,6 +53,16 @@ void knetfilter::addRule() {
     addp.clearArguments();
     addp2.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -72,16 +88,6 @@ void knetfilter::addRule() {
     if(sport.length()==0 && dport.length()==0) 
       QMessageBox::critical(0, "Error",
     			"You have to supply a source or destination port number.\n");
-    else if(ip.length()==0 && netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address,\n"
-			"and a netmask address.\n");
-    else if(ip.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address.\n");
-    else if(netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a netmask address.\n");
     else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
       QMessageBox::critical(0, "Error",
       			"Not a valid IP address and Netmask address.\n");
@@ -115,22 +121,33 @@ void knetfilter::addRule() {
       }
       else
 	{
+	  if (strcmp("unclean",stateChoice)==0)
+	    {
+	      state_modules = "unclean";
+	      state_definition = "-v";
+	      stateChoice = "-v";
+	    }
+	  else
+	    {
+	      state_modules = "state"; 
+	      state_definition = "--state";
+	    }
 	  if (dport.length()==0)
 	    {
-	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-	      addp2 << "iptables" << ruleaction << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      addp2 << "iptables" << ruleaction << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 	    }
 	  else
 	    {
 	      if (sport.length()==0)
 		{
-		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		}
 	      else
 		{
-		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		}
 	    }
 	}
@@ -153,9 +170,15 @@ void knetfilter::addRule_tcp() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("255.255.255.255");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
 	extramask.sprintf("255.255.255.255");
@@ -171,6 +194,16 @@ void knetfilter::addRule_tcp() {
 
     addp.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -195,16 +228,6 @@ void knetfilter::addRule_tcp() {
     if(sport.length()==0 && dport.length()==0 ) 
       QMessageBox::critical(0, "Error",
     			"You have to supply a source or destination port number.\n");
-    else if(ip.length()==0 && netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address,\n"
-			"and a netmask address.\n");
-    else if(ip.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address.\n");
-    else if(netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a netmask address.\n");
     else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
       QMessageBox::critical(0, "Error",
       			"Not a valid IP address and Netmask address.\n");
@@ -235,19 +258,30 @@ void knetfilter::addRule_tcp() {
 	}
       else
 	{
+	  if (strcmp("unclean",stateChoice)==0)
+	    {
+	      state_modules = "unclean";
+	      state_definition = "-v";
+	      stateChoice = "-v";
+	    }
+	  else
+	    {
+	      state_modules = "state"; 
+	      state_definition = "--state";
+	    }
 	  if (dport.length()==0)
 	    {
-	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+	      addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 	    }
 	  else
 	    {
 	      if (sport.length()==0)	
 		{
-		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
 	      else
 		{
-		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		}
 	    }      	  
 	}	
@@ -269,9 +303,15 @@ void knetfilter::addRule_udp() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("255.255.255.255");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
 	extramask.sprintf("255.255.255.255");
@@ -286,6 +326,16 @@ void knetfilter::addRule_udp() {
 
     addp2.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -310,16 +360,6 @@ void knetfilter::addRule_udp() {
     if(sport.length()==0 && dport.length()==0) 
       QMessageBox::critical(0, "Error",
     			"You have to supply a source or detination port number.\n");
-    else if(ip.length()==0 && netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address,\n"
-			"and a netmask address.\n");
-    else if(ip.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address.\n");
-    else if(netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a netmask address.\n");
     else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
       QMessageBox::critical(0, "Error",
       			"Not a valid IP address and Netmask address.\n");
@@ -350,19 +390,30 @@ void knetfilter::addRule_udp() {
 	}
       else
 	{
+	  if (strcmp("unclean",stateChoice)==0)
+	    {
+	      state_modules = "unclean";
+	      state_definition = "-v";
+	      stateChoice = "-v";
+	    }
+	  else
+	    {
+	      state_modules = "state"; 
+	      state_definition = "--state";
+	    }
 	  if (dport.length()==0)
 	    {
-	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+	      addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 	    }
 	  else
 	    {
 	      if (sport.length()==0)	
 		{
-		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
 	      else
 		{
-		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  addp2 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		}
 	    }
 	}
@@ -385,7 +436,12 @@ void knetfilter::addRule_icmp() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("32");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
     if(extramask.length()==0)
@@ -401,6 +457,16 @@ void knetfilter::addRule_icmp() {
 
     addp3.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -425,16 +491,6 @@ void knetfilter::addRule_icmp() {
     if(type.length()==0) 
       QMessageBox::critical(0, "Error",
     			"You have to supply an icmp type number.\n");
-    else if(ip.length()==0 && netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address,\n"
-			"and a netmask address.\n");
-    else if(ip.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a IP address.\n");
-    else if(netmask.length()==0)
-      QMessageBox::critical(0, "Error",
-      			"You have to supply a netmask address.\n");
     else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
       QMessageBox::critical(0, "Error",
       			"Not a valid IP address and Netmask address.\n");
@@ -451,7 +507,18 @@ void knetfilter::addRule_icmp() {
 	}
       else
 	{
-	  addp3 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << "state" << "--state" << stateChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	  if (strcmp("unclean",stateChoice)==0)
+	    {
+	      state_modules = "unclean";
+	      state_definition = "-v";
+	      stateChoice = "-v";
+	    }
+	  else
+	    {
+	      state_modules = "state"; 
+	      state_definition = "--state";
+	    }
+	  addp3 << "iptables" << ruleaction << chainChoice << rulepos << "-m" << state_modules << state_definition << stateChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 	}
     };
 
@@ -471,9 +538,15 @@ void knetfilter::delRule() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("255.255.255.255");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
 	extramask.sprintf("255.255.255.255");
@@ -491,6 +564,16 @@ void knetfilter::delRule() {
     delp.clearArguments();
     delp2.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -511,22 +594,12 @@ void knetfilter::delRule() {
 	if(sport.length()==0 && dport.length()==0) 
 	  QMessageBox::critical(0, "Error",
 				"You have to enter a source or destination port number\n");
-	else if(ip.length()==0 && netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address,\n"
-				"and a netmask address.\n");
-	else if(ip.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address.\n");
-	else if(netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a netmask address.\n");
 	else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
 	  QMessageBox::critical(0, "Error",
 				"Not a valid IP address and Netmask address.\n");
-    else if(!strcmp(ip,"Couldn't probe"))
-      QMessageBox::critical(0, "Error",
-			    "Not a valid IP address.\n");
+	else if(!strcmp(ip,"Couldn't probe"))
+	  QMessageBox::critical(0, "Error",
+				"Not a valid IP address.\n");
 	else if(!strcmp(netmask,"Couldn't probe"))
 	  QMessageBox::critical(0, "Error",
 				"Not a valid netmask address.\n");
@@ -554,22 +627,33 @@ void knetfilter::delRule() {
 	    }
 	  else
 	    {
+	      if (strcmp("unclean",stateChoice)==0)
+		{
+		  state_modules = "unclean";
+		  state_definition = "-v";
+		  stateChoice = "-v";
+		}
+	      else
+		{
+		  state_modules = "state"; 
+		  state_definition = "--state";
+		}
 	      if (dport.length()==0)
 		{
-		  delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		  delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  delp << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		  delp2 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		}
 	      else
 		{
 		  if (sport.length()==0)
 		    {
-		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		    }
 		  else
 		    {
-		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
-		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		    }
 		}
 	    }
@@ -594,9 +678,15 @@ void knetfilter::delRule_tcp() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("255.255.255.255");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
 	extramask.sprintf("255.255.255.255");
@@ -613,6 +703,16 @@ void knetfilter::delRule_tcp() {
 
     delp.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -633,16 +733,6 @@ void knetfilter::delRule_tcp() {
 	if(sport.length()==0 && dport.length()==0) 
 	  QMessageBox::critical(0, "Error",
 				"You have to enter a source or destination port number\n");
-	else if(ip.length()==0 && netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address,\n"
-				"and a netmask address.\n");
-	else if(ip.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address.\n");
-	else if(netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a netmask address.\n");
 	else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
 	  QMessageBox::critical(0, "Error",
 				"Not a valid IP address and Netmask address.\n");
@@ -673,19 +763,30 @@ void knetfilter::delRule_tcp() {
 	    }
 	  else
 	    {
+	      if (strcmp("unclean",stateChoice)==0)
+		{
+		  state_modules = "unclean";
+		  state_definition = "-v";
+		  stateChoice = "-v";
+		}
+	      else
+		{
+		  state_modules = "state"; 
+		  state_definition = "--state";
+		}
 	      if (dport.length()==0)
 		{
-		  delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		  delp << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
 	      else
 		{
 		  if (sport.length()==0)	
 		    {
-		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		      delp << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		    }
 		  else
 		    {
-		      delp << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "tcp" << flagChoice << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		    }
 		}      
 	    }
@@ -710,9 +811,15 @@ void knetfilter::delRule_udp() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("255.255.255.255");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
 	extramask.sprintf("255.255.255.255");
@@ -729,6 +836,16 @@ void knetfilter::delRule_udp() {
 
     delp2.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -748,16 +865,6 @@ void knetfilter::delRule_udp() {
 	if(sport.length()==0 && dport.length()==0) 
 	  QMessageBox::critical(0, "Error",
 				"You have to enter a source or destination port number\n");
-	else if(ip.length()==0 && netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address,\n"
-				"and a netmask address.\n");
-	else if(ip.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address.\n");
-	else if(netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a netmask address.\n");
 	else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
 	  QMessageBox::critical(0, "Error",
 				"Not a valid IP address and Netmask address.\n");
@@ -788,19 +895,30 @@ void knetfilter::delRule_udp() {
 	    }
 	  else
 	    {
+	      if (strcmp("unclean",stateChoice)==0)
+		{
+		  state_modules = "unclean";
+		  state_definition = "-v";
+		  stateChoice = "-v";
+		}
+	      else
+		{
+		  state_modules = "state"; 
+		  state_definition = "--state";
+		}
 	      if (dport.length()==0)
 		{
-		  delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		  delp2 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		}
 	      else
 		{
 		  if (sport.length()==0)	
 		    {
-		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice; 
 		    }
 		  else
 		    {
-		      delp2 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+		      delp2 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "udp" << "--sport" << sport << "--dport" << dport << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 		    }
 		}
 	    }
@@ -825,12 +943,18 @@ void knetfilter::delRule_icmp() {
     ip.sprintf((const char *)(ipAddr->text()));
     netmask.sprintf((const char *)(netmaskAddr->text()));
 
-    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmaskAddr->text()));
+    if(netmask.length()==0)
+      {
+	netmask.sprintf("32");
+      }
+
+    network.sprintf("%s/%s",(const char *)(ipAddr->text()), (const char *)(netmask));
 
     extramask.sprintf((const char *)(extraNetMask->text()));
+
     if(extramask.length()==0)
       {
-	extramask.sprintf("255.255.255.0");
+	extramask.sprintf("32");
       }
 
     extranetwork.sprintf((const char *)(extraNet->text()));
@@ -843,6 +967,16 @@ void knetfilter::delRule_icmp() {
 
     delp3.clearArguments();
 
+    if(ip.length()==0)
+      {
+	network.sprintf("0.0.0.0/0");
+      }
+
+    if((strcmp((const char *)(ipAddr->text()),"0.0.0.0")==0))
+    {      
+      network.sprintf("0.0.0.0/0");
+    }
+
     if(extranetwork.length()==0)
       {
 	extranet.sprintf("0.0.0.0/0");
@@ -862,16 +996,6 @@ void knetfilter::delRule_icmp() {
 	if(type.length()==0) 
 	  QMessageBox::critical(0, "Error",
 				"You have to enter an icmp type number\n");
-	else if(ip.length()==0 && netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address,\n"
-				"and a netmask address.\n");
-	else if(ip.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a IP address.\n");
-	else if(netmask.length()==0)
-	  QMessageBox::critical(0, "Error",
-				"You have to supply a netmask address.\n");
 	else if(!strcmp(ip,"Couldn't probe") && !strcmp(netmask,"Couldn't probe"))
 	  QMessageBox::critical(0, "Error",
 				"Not a valid IP address and Netmask address.\n");
@@ -888,7 +1012,18 @@ void knetfilter::delRule_icmp() {
 	    }
 	  else
 	    {
-	      delp3 << "iptables" << "-D" << chainChoice << "-m" << "state" << "--state" << stateChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
+	      if (strcmp("unclean",stateChoice)==0)
+		{
+		  state_modules = "unclean";
+		  state_definition = "-v";
+		  stateChoice = "-v";
+		}
+	      else
+		{
+		  state_modules = "state"; 
+		  state_definition = "--state";
+		}
+	      delp3 << "iptables" << "-D" << chainChoice << "-m" << state_modules << state_definition << stateChoice << "-p" << "icmp" << "--icmp-type" << type << "-s" << extranet << "-d" << network << "-j" << policyChoice;
 	    }
 	}
       };
@@ -948,7 +1083,7 @@ void knetfilter::saveAll() {
 
   save.clearArguments();
 
-  /*  FILE *cfgfile;
+  FILE *cfgfile;
   FILE *saver1;
   FILE *saver2;
   FILE *saver3;
@@ -979,7 +1114,7 @@ void knetfilter::saveAll() {
   if ( saver1==NULL && saver2==NULL && saver3==NULL && saver4==NULL && saver5==NULL && saver6==NULL || restore1==NULL && restore2==NULL && restore3==NULL && restore4==NULL && restore5==NULL && restore6==NULL )
     {
       QMessageBox::information(this, "Warning",
-			      "Couldn't locate iptables-save binary from iptables-1.2 or newer.\n"
+			      "Couldn't locate iptables-save binary from iptables-1.2.1a or newer.\n"
 			      "For now I will use my own save function, but, please, consider\n"
 			      "to upgrade your iptables as soon, enabling it's own\n"
 			      "iptables-save and iptables-restore command lines. (SEE iptables README)."); 
@@ -993,10 +1128,10 @@ void knetfilter::saveAll() {
 	  remove("/etc/iptables.cfg");
 	}
       save  << "kiptables-save" << "iptorig";  
-      } */
+    } 
 
   // to be removed when iptables-save will work
-  save << "kiptables-save";
+  // save << "kiptables-save";
 
   bool success=save.start();
 
@@ -1026,7 +1161,7 @@ void knetfilter::loadAll() {
   load.clearArguments();
   
 
-  /*  FILE *cfgfile_2;
+  FILE *cfgfile_2;
   FILE *saver1_2;
   FILE *saver2_2;
   FILE *saver3_2;
@@ -1065,7 +1200,7 @@ void knetfilter::loadAll() {
       if ( saver1_2==NULL && saver2_2==NULL && saver3_2==NULL && saver4_2==NULL && saver5_2==NULL && saver6_2==NULL || restore1_2==NULL && restore2_2==NULL && restore3_2==NULL && restore4_2==NULL && restore5_2==NULL && restore6_2==NULL )
 	{
 	  QMessageBox::information(this, "Warning",
-				   "Couldn't locate iptables-save binary from iptables-1.2 or newer.\n"
+				   "Couldn't locate iptables-save binary from iptables-1.2.1a or newer.\n"
 				   "For now I will use my own save function, but, please, consider\n"
 				   "to upgrade your iptables as soon, enabling it's own\n"
 				   "iptables-save and iptables-restore command lines. (SEE iptables README)."); 
@@ -1073,12 +1208,15 @@ void knetfilter::loadAll() {
 	}
       else
 	{
-	  load << "kiptables-restore" << "iptorig"; 
+	  load << "kiptables-restore"; 
+	  // not needed since kiptables-save script is smarter than i thought
+	  // anyway, i want to advise if users should upgrade
+	  //	  load << "kiptables-restore" << "iptorig"; 
 	}
-	}*/
+    }
 
   // to be removed when iptables-restore will work
-  load << "kiptables-restore"; 
+  // load << "kiptables-restore"; 
 
   bool success=load.start();
 
